;; Generated package description from auctex.el  -*- no-byte-compile: t -*-
(define-package "auctex" "12.3.1.0.20201220.154121" "Integrated environment for *TeX*" '((emacs "24.1") (cl-lib "0.5")) :keywords '("tex" "latex" "texinfo" "context" "doctex" "preview-latex") :maintainer '(nil . "auctex-devel@gnu.org") :url "http://www.gnu.org/software/auctex/")
